/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.navigator;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.fortiss.tooling.kernel.interfaces.IModelElementHandler;
import org.fortiss.tooling.kernel.interfaces.ITopLevelElementContext;
import org.fortiss.tooling.kernel.services.IModelElementService;
import org.fortiss.tooling.kernel.services.IPersistencyService;

/**
 * Content provider for the model navigator tree viewer.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: DBA3E3CC4704ECB40F35CF9D36BFE89A
 */
public class NavigatorTreeContentProvider implements ITreeContentProvider {

	/** {@inheritDoc} */
	@Override
	public Object[] getElements(Object inputElement) {
		// delegate to the top-level element contexts of the persistency service
		List<EObject> result = new LinkedList<EObject>();

		for (ITopLevelElementContext context : IPersistencyService.INSTANCE
				.getTopLevelElementContexts()) {
			result.add(context.getTopLevelElement());
		}
		return result.toArray();
	}

	/** {@inheritDoc} */
	@Override
	public Object[] getChildren(Object parentElement) {
		// delegate to the model element handlers
		if (parentElement instanceof EObject) {
			IModelElementHandler<EObject> handler = IModelElementService.INSTANCE
					.getModelElementHandler((EObject) parentElement);
			if (handler != null) {
				return handler.getSubnodes((EObject) parentElement).toArray();
			}
		}
		return new Object[0];
	}

	/** {@inheritDoc} */
	@Override
	public Object getParent(Object element) {
		if (element instanceof EObject
				&& IPersistencyService.INSTANCE
						.isTopLevelElement((EObject) element)) {
			// delegate to persistency service
			return IPersistencyService.INSTANCE;
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean hasChildren(Object element) {
		return getChildren(element).length > 0;
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		// nothing to do
	}

	/** {@inheritDoc} */
	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// the input of this viewer is the persistency service singleton and
		// therefore never changes.
	}
}
